from unittest import TestCase

from .context import ImageBuilder


class ImageTest(TestCase):

    def test_file(self):
        file = open("resources/image.jpeg", "rb")
        builder = ImageBuilder("image.jpeg", file.read())
        image = builder.image
        self.assertIsNotNone(image)



from datetime import date, datetime
from uuid import uuid4

from logging_utils import LogDecorator

from ..loader.image import Image


class Pipeline:

    objects = []

    def __init__(self, image: Image) -> None:
        """
        Une pipeline est une succession d'étape

        """
        self.__pipeline_id = str(uuid4())
        self.__status = 'CREATED'
        self.__start_time = datetime.now()
        self.__end_time = None
        self.__image = image
        self.__steps = None
        self.__class__.objects.append(self)

    @staticmethod
    def find_by_id(pipeline_id):
        try:
            return next(item for item in Pipeline.objects if item.pipeline_id == pipeline_id)
        except StopIteration:
            return None

    @LogDecorator('pipeline_del_inspect')
    def __del__(self):
        self.__class__.objects.remove(self)

    def __dict__(self):
        return {'pipeline_id': self.__pipeline_id, 'start_time': self.__start_time.strftime("%m/%d/%Y, %H:%M:%S"),
                'end_time': self.__end_time.strftime("%m/%d/%Y, %H:%M:%S") if self.__end_time else None,
                'status': self.__status, 'steps': [step.__dict__() for step in self.__steps]}

    def stop_pipeline(self):
        self.__end_time = datetime.now()
        self.__status = 'FINISHED'

    @property
    def pipeline_id(self):
        return self.__pipeline_id

    @property
    def steps(self) -> list:
        return self.__steps

    @steps.setter
    def steps(self, values:list) -> None:
        self.__steps = values

    @property
    def image(self) -> Image:
        return self.__image

    @image.setter
    def image(self, value) -> None:
        self.__image = value


class Step:

    objects = []

    def __init__(self, name):
        self.__step_id = str(uuid4())
        self.__status = 'CREATED'
        self.__start_time = None
        self.__end_time = None
        self.__name = name
        self.__coid = None
        self.__class__.objects.append(self)

    @staticmethod
    def find_by_id(step_id):
        try:
            return next(item for item in Step.objects if item.step_id == step_id)
        except StopIteration:
            return None

    @LogDecorator('step_del_inspect')
    def __del__(self):
        self.__class__.objects.remove(self)

    @property
    def status(self):
        return self.__status

    @property
    def step_id(self):
        return self.__step_id

    @property
    def name(self):
        return self.__name

    def launch_job(self):
        self.__start_time = datetime.now()
        self.__status = 'IN PROGRESS'

    def stop_job(self, consumer_id):
        self.__end_time = datetime.now()
        self.__coid = consumer_id
        self.__status = 'FINISHED'

    def __dict__(self):
        return {'step_id': self.__step_id, 'name': self.__name, 'consumer': self.__coid,
                'start_time': self.__start_time.strftime("%m/%d/%Y, %H:%M:%S") if self.__start_time else None,
                'end_time': self.__end_time.strftime("%m/%d/%Y, %H:%M:%S") if self.__end_time else None,
                'status': self.__status}


class PipelineBuilder:

    def __init__(self) -> None:
        self.__pipeline = None

    def __reset(self):
        self.__pipeline = None

    def create(self, image: Image) -> None:
        self.__pipeline = Pipeline(image)
        self.__pipeline.steps = [Step(operation) for operation in image.operations]

    @property
    def pipeline(self):
        pipeline = self.__pipeline
        self.__reset()
        return pipeline



from datetime import date

import tornado
from logging_utils import Chronometer, LogDecorator
from tornado.web import MissingArgumentError

from .image import ImageBuilder
from .rabbit import Messaging
from ..pipelining import PipelineBuilder

messenger = Messaging()


class UploadHandler(tornado.web.RequestHandler):

    def data_received(self, chunk):
        pass

    @Chronometer(function_name='upload-post-timer')
    @LogDecorator(decorator_log='upload-post-inspect')
    def post(self):  # POST /upload

        try:
            tags = self.get_argument('tags')
            if tags:
                tags = tags.split(',')
            else:
                tags = []
        except MissingArgumentError:
            tags = []

        try:
            operations = self.get_argument('operations')
            if operations:
                operations = operations.split(',')
            else:
                operations = []
        except MissingArgumentError:
            operations = []

        print(f"nombre d'images {len(self.request.files)}")
        for _, files in self.request.files.items():
            for info in files:
                filename = info["filename"]
                body = info["body"]

                # Création de l'image
                builder = ImageBuilder(filename, body)
                builder.operations(operations)
                builder.tags(tags)
                image = builder.image

                if image.validate():
                    # Création de la pipeline associée
                    pipeline_builder = PipelineBuilder()
                    pipeline_builder.create(image)
                    pipeline = pipeline_builder.pipeline
                    messenger.send_to_pipeline(pipeline)
                    self.finish({'image': image.rename_file, 'operations': operations})
                else:
                    self.set_status(400)
                    self.finish({'error': 'bad request', 'reason': 'One or more consumers are unknown.'})



from datetime import date
from uuid import uuid4

from logging_utils import LogDecorator

from core.consumer.api import Consumer


class Image:

    def __init__(self, content, filename, rename_file, extension, date_creation):
        self.__content = content
        self.__filename = filename
        self.__rename_file = rename_file
        self.__extension = extension
        self.__date_creation = date_creation
        self.__operations = None
        self.__tags = None

    @property
    def tags(self) -> list:
        return self.__tags

    @tags.setter
    def tags(self, tags: list):
        self.__tags = tags

    @property
    def date_create(self):
        return self.__date

    @property
    def operations(self) -> list:
        return self.__operations

    @operations.setter
    def operations(self, operations: list):
        self.__operations = operations

    @property
    def rename_file(self) -> str:
        return self.__rename_file

    @property
    def extension(self) -> str:
        return self.__extension

    @property
    def content(self):
        return self.__content

    @LogDecorator('image-validate-inspect')
    def validate(self):
        # @TODO La validation des consommateurs n'est pas de la responsabilité de l'image
        consumers = [Consumer.find_by_name(operation) for operation in self.__operations]
        return not any(consumer is None for consumer in consumers)


class ImageBuilder:

    def __init__(self, filename, body):
        # Get extension
        extension = filename.split('.')[-1]
        rename = f'{str(uuid4())}.{extension}'
        self.__image = Image(body, filename, rename, extension, date.today())

    def operations(self, operations:list) -> None:
        self.__image.operations = operations

    def tags(self, tags:list) -> None:
        self.__image.tags = tags

    @property
    def image(self) -> Image:
        return self.__image



import logging
import os

import pika
from logging_utils import Chronometer, LogDecorator

RABBIT_HOST = os.getenv('RABBIT_HOST', '127.0.0.1')


class Messaging:

    @staticmethod
    def __create_connection():
        return pika.BlockingConnection(pika.ConnectionParameters(
            host=RABBIT_HOST
        ))

    def __init__(self):
        self.__log = logging.getLogger('Messaging')

    def __send_message(self, queue_name, image, pipeline_id, step_id):
        connexion = Messaging.__create_connection()

        channel = connexion.channel()
        channel.queue_declare(queue=queue_name)

        properties = {'tags': image.tags, 'operations': image.operations, 'pipeline': pipeline_id, 'step': step_id}

        channel.basic_publish(
            exchange='',
            routing_key=queue_name,
            body=image.content,
            properties=pika.BasicProperties(headers=properties)
        )

        self.__log.debug(f'[x] image sent to {queue_name}')

        connexion.close()

        return queue_name

    @Chronometer(function_name='pipeline-send-timer')
    @LogDecorator(decorator_log='pipeline-send-inspect')
    def send_to_pipeline(self, pipeline):
        # Get Next step
        try:
            next_step = next(x for x in pipeline.steps if x.status == 'CREATED')
        except StopIteration:
            self.__log.debug('No more step found, pipeline ending')
            pipeline.stop_pipeline()
            return

        next_step.launch_job()
        self.__send_message(next_step.name, pipeline.image, pipeline.pipeline_id, next_step.step_id)

    @staticmethod
    def get_consumers(queue_name) -> int:
        connexion = Messaging.__create_connection()

        channel = connexion.channel()
        queue_state = channel.queue_declare(queue=queue_name)

        return queue_state.method.consumer_count



import tornado
from logging_utils import Chronometer, LogDecorator
from tornado.web import MissingArgumentError

from .pipeline import Pipeline, Step
from ..loader.image import ImageBuilder
from ..loader.rabbit import Messaging

messenger = Messaging()


class StepHandler(tornado.web.RequestHandler):

    def data_received(self, chunk):
        pass

    @Chronometer(function_name='finish-post-timer')
    @LogDecorator(decorator_log='finish-post-inspect')
    def post(self):  # POST /finish
        # Get parameters
        try:
            tags = self.get_argument('tags')
            if tags:
                tags = tags.split(',')
            else:
                tags = []
        except MissingArgumentError:
            tags = []

        operations = self.get_argument('operations')
        pipeline_id = self.get_argument('pipeline_id')
        pipeline = Pipeline.find_by_id(pipeline_id)

        step_id = self.get_argument('step_id')
        step = Step.find_by_id(step_id)

        consumer_id = self.get_argument('consumer_id')

        step.stop_job(consumer_id)

        for _, files in self.request.files.items():
            for info in files:
                filename = info["filename"]
                body = info["body"]

                builder = ImageBuilder(filename, body)
                builder.operations(operations)
                builder.tags(tags)
                image = builder.image

                pipeline.image = image

                messenger.send_to_pipeline(pipeline)


class AdminStepHandler(tornado.web.RequestHandler):

    def data_received(self, chunk):
        pass

    @Chronometer(function_name='admin-pipeline-get-timer')
    @LogDecorator(decorator_log='admin-pipeline-get-inspect')
    def get(self, slug):  # GET /admin/pipelines/_slug_
        if slug == 'list':
            self.set_status(200)
            self.finish({'nb_pipeline': len(Pipeline.objects),
                         'pipelines': [item.__dict__() for item in Pipeline.objects]})
        else:
            pipeline = Pipeline.find_by_oid(slug)
            if not pipeline:
                self.set_status(404)
                self.finish({'error': 'not found', 'reason': "no pipeline exists"})
            else:
                self.set_status(200)
                self.finish(pipeline.__dict__())